--[[
This file contains a table which defines options available to the user in the options dialog

tab data is what populates each tab and defines each option in a tab
Each tab has:
    .title - the text that will appear on the tab
    .key - the key that will group the options (ie in the prefs file gameplay.help_prompts or video.resolution)
    .items - an array of the items for this key (this is an array rather than a genericly keyed table so display order can be imposed)
        .title - the text that will label the option line item
        .tip - the text that will appear in the tooltip for the item
        .key - the prefs key to identify this property
        .default - the default value of the property
        .restart - if true, setting the option will require a restart and the user will be notified
        .verify - if true, prompts the user to veryfiy the change for 15 seconds, otherwise defaults back to prior setting
        .populate - an optional function which when called, will repopulate the options custom data. The value passed in is the current value of the control (function(value))
        .set - an optional function that takes a value parameter and is responsible for determining what happens when the option is applied (function(key, value))
        .ignore - an optional function called when the option is set, checks the value, and won't change it from its former setting, and if former setting is invalid, uses return value for new value (function(value))
        .cancel - called when the option is cancelled
        .beginChange - an option function for sliders when user begins modification
        .endChange - an option function for sliders when user ends modification
        .update - a optional function that is called when the control has a new value, also receives the control (function(control, value)), not always used,
                  but if you need additonal control (say of other controls) when this value changes (for example one control may change other controls) you can
                  set up that behavior here
        .type - the type of control used to display this property
            valid types are:
            toggle - multi state toggle button (TODO - add list to replace more than 2 states?)
            button - momentary button (usually open another dialog)
            slider - a value slider
        .custom - a table of data required by the control type, different for each control type.
            slider
                .min - the minimum value for the slider
                .max - the maximum value for the slider
                .inc - the increment between slider detents, if 0 its "analog"
            toggle
                .states - table of states the toggle switch can have
                    .text = text for each state
                    .key = the key or value for each state to be stored in the pref
            button
                .text - the text label of the button

the optionsOrder table is just an array of keys in to the option table, and their order will determine what
order the tabs show in the dialog
--]]




optionsOrder = {
    "gameplay",
    "video",
    "sound",
  	--"about",
    "mods",
}

if not options.mods then
	options.mods = {
	    title = "Mods",
	    key = 'mods',
	    items = { }
	}
end


table.insert(options.mods.items, {
    title = "Show Stragic Icons",
    key = 'oi_straticons_activated',
    type = 'toggle',
    default = 1,
    custom = {
        states = {
            {text = "<LOC _Off>", key = 0 },
            {text = "<LOC _On>", key = 1 },
        },
    }
})

table.insert(options.mods.items, {
    title = "Strategic Icon Set",
    key = 'oi_straticons_set',
    type = 'toggle',
    default = 'default',
    custom = {
        states = {
            {text = "()((()", key = 'default' },
            {text = "III_Demon", key = '3demon' },
        },
    }
})

table.insert(options.mods.items,  {
    title = "Strategic Icon Opacity",
    key = 'oi_straticons_opacity',
    type = 'slider',
    default = 75,
    custom = {
        min = 0,
        max = 100,
        inc = 1,
    },
})

table.insert(options.mods.items,  {
    title = "Show Abilities in Unitview (*)",
    key = 'oi_unitview_abilities',
    type = 'toggle',
    default = 0,
    custom = {
        states = {
            {text = "<LOC _Off>", key = 0 },
            {text = "<LOC _On>", key = 1 },
        },
    }
})

table.insert(options.mods.items,  {
    title = "Build Modifier SHIFT",
    key = 'oi_build_shiftvalue',
    type = 'slider',
    default = 5,
    custom = {
        min = 1,
        max = 100,
        inc = 1,
    },
})

table.insert(options.mods.items,  {
    title = "Build Modifier CTRL",
    key = 'oi_build_ctrlvalue',
    type = 'slider',
    default = 5,
    custom = {
        min = 1,
        max = 100,
        inc = 1,
    },
})

table.insert(options.mods.items,  {
    title = "Build Modifier CTRL+SHIFT",
    key = 'oi_build_ctrlshiftvalue',
    type = 'slider',
    default = 5,
    custom = {
        min = 1,
        max = 100,
        inc = 1,
    },
})

table.insert(options.mods.items,  {
    title = "Reverse Tech Level Switcher",
    key = 'oi_build_reversetechswitcher',
    type = 'toggle',
    default = 0,
    custom = {
        states = {
            {text = "<LOC _Off>", key = 0 },
            {text = "<LOC _On>", key = 1 },
        },
    }
})



