
local OIUtil = import('/mods/OllosInterface/libs/oiutil.lua')

function CreateAvatarIcons(parent)
    avatarContainer = Group(parent, "avatarContainer")
    avatarContainer.Top:Set(function() return parent.Top() + 20 end)
    avatarContainer.Left:Set(parent.Left)
    avatarContainer.Right:Set(function() return parent.Right() - 5 end)
    avatarContainer.Bottom:Set(function() return parent.Bottom() - 55 end)
    avatarContainer:DisableHitTest()
    avatarContainer:SetNeedsFrameUpdate(true)
    
    avatarToggle = UIUtil.CreateCheckbox(avatarContainer, 
        OIUtil.getImage('toggleavatar_on'),
        OIUtil.getImage('toggleavatar_off'),
        OIUtil.getImage('toggleavatar_on_mo'),
        OIUtil.getImage('toggleavatar_off_mo'),
        OIUtil.getImage('blank'),
        OIUtil.getImage('blank'), 
        'UI_Avatar_Hide')
    LayoutHelpers.AtRightTopIn(avatarToggle, parent, 0, 0)
    avatarToggle.OnCheck = function(self, checked)
        ToggleAvatars(not checked)
    end
    ToolTip.AddCheckboxTooltip(avatarToggle, 'avatar_toggle')
    
    GameMain.AddBeatFunction(AvatarUpdate)
end

function LayoutSort(Type, Index)
    local index = Index
    for i, v in avatars do
        local tempindex = index
        if v.Type == Type then
            if index == 1 then
                v.icon.Right:Set(avatarContainer.Right)
                v.icon.Top:Set(avatarContainer.Top)
            else
            	--()((()-- scoreTop is not feasable
                --local scoreTop = import('/lua/ui/game/score.lua').GetScoreTop() or avatarContainer.Bottom()
                scoreTop = avatarContainer.Bottom()
                --()((()--
                if v.icon.Height() + avatarKey[index-1].icon.Bottom() > scoreTop then
                    v.icon.Right:Set(function() return avatarKey[tempindex-1].icon.Left() - 19 end)
                    v.icon.Top:Set(avatarContainer.Top)
                else
                    v.icon.Right:Set(avatarKey[index-1].icon.Right)
                    v.icon.Top:Set(function() return avatarKey[tempindex-1].icon.Bottom() + 2 end)
                end
            end
            avatarKey[index] = v
            index = index + 1
        end
    end
    return index
end

function CreateAvatar(Data, Index)
    local bg = Button(avatarContainer,
        OIUtil.getImage('avatar_bg'),
        OIUtil.getImage('avatar_bg'),
        OIUtil.getImage('avatar_bg'),
        OIUtil.getImage('avatar_bg')
    )
    
    bg.UnitIndex = 1
    bg.Units = Data.Units
    local function CommonHandler(control, event, data)
        if event.Type == 'MouseEnter' then
            CreateMouseover(control, data)
        elseif event.Type == 'MouseExit' then
            CloseMouseover()
        end
    end
    if Data.Type == 'Avatar' or Data.Type == 'Engineer' then
        bg.HandleEvent = function(self, event)
            if event.Type == "ButtonPress" or event.Type == "ButtonDClick" then
                if self.Units[self.UnitIndex + 1] then
                    self.UnitIndex = self.UnitIndex + 1
                else
                    self.UnitIndex = 1
                end
                if event.Modifiers.Left then
                    if event.Type == "ButtonPress" then
                        SelectUnits({self.Units[self.UnitIndex]})
                    else
                        SelectUnits(self.Units)
                    end
                elseif event.Modifiers.Right then
                    UISelectAndZoomTo(self.Units[self.UnitIndex])
                end
            end
            CommonHandler(self, event, Data)
            return true
        end
    elseif Data.Type == 'Group' then
        bg.HandleEvent = function(self, event)
            if event.Type == "ButtonPress" or event.Type == "ButtonDClick" then
                if event.Modifiers.Left then
                    if self.Units[self.UnitIndex + 1] then
                        self.UnitIndex = self.UnitIndex + 1
                    else
                        self.UnitIndex = 1
                    end
                    if event.Type == "ButtonPress" then
                        SelectUnits(self.Units)
                    elseif event.Type == "ButtonDClick" then
                        UISelectAndZoomTo(self.Units[self.UnitIndex])
                        SelectUnits(self.Units)
                    end
                end
            end
            CommonHandler(self, event, Data)
            return true
        end
    end

    local icon = Bitmap(bg, GetAvatarIcon(Data))
    icon.Width:Set(46)
    icon.Height:Set(46)
    LayoutHelpers.AtLeftTopIn(icon, bg, 2, 2)

    if Data.Type == 'Avatar' then
        local healthbar = StatusBar(bg, 0, 1, false, false,
            UIUtil.UIFile('/game/avatar/health-bar-back_bmp.dds'),
            UIUtil.UIFile('/game/avatar/health-bar-green.dds'),
            true, "avatar RO Health Status Bar")
        healthbar.Left:Set(function() return bg.Left() + 6 end)
        healthbar.Right:Set(function() return bg.Right() - 6 end)
        healthbar.Bottom:Set(function() return bg.Bottom() - 5 end)
        healthbar.Top:Set(function() return healthbar.Bottom() - 10 end)
        healthbar.Height:Set(function() return healthbar.Bottom() - healthbar.Top() end)
        healthbar.Width:Set(function() return healthbar.Right() - healthbar.Left() end)
        healthbar:DisableHitTest(true)
        
        local alertSound = {}
        local name = ""
        local blueprintID = Data.Units[1]:GetBlueprint().BlueprintId
        local faction = string.sub(blueprintID, 2, 2)

        local factionData = import('/lua/factions.lua')
        local facKey = factionData.FactionDesToKey[string.upper(faction)]
        
        name = factionData.Factions[factionData.FactionIndexMap[facKey]].SoundPrefix

        alertSound = Sound {
            Bank = string.format('COMPUTER_%s_VO', string.upper(name)),
            Cue = string.format("%sComputer_Commanders_01314",name),
        }
        
        local glow = Bitmap(bg, OIUtil.getImage('avatar_glow'))
        LayoutHelpers.AtLeftTopIn(glow, bg, 0, 0)
        glow.Depth:Set(function() return bg.Depth() + 1 end)
        glow:DisableHitTest()
        glow.Time = 0
        glow.PhaseTime = 0
        glow:SetAlpha(0)
        glow.OnFrame = function(self, delta)
            if self.Time > 5 and self:GetAlpha() < .3 then
                self:SetAlpha(0)
                self:SetNeedsFrameUpdate(false)
            end
            self.Time = self.Time + delta
            self.PhaseTime = self.PhaseTime + delta
            self:SetAlpha(math.abs(math.sin(self.PhaseTime * 2)))
        end
        
        bg.Update = function(Data)
            local tempPrevHealth = healthbar._value()
            local tempHealth = Data.Units[1]:GetHealth()
            if tempPrevHealth > tempHealth then
                if not glow:NeedsFrameUpdate() then
                    PlayVoice(alertSound)
                    glow:SetNeedsFrameUpdate(true)
                    glow.PhaseTime = 0
                end
                glow.Time = 0
            end
            healthbar:SetRange(0, Data.Units[1]:GetMaxHealth())
            healthbar:SetValue(tempHealth)
            if tempPrevHealth != tempHealth then
                SetHealthbarColor(healthbar, Data.Units[1]:GetHealth() / Data.Units[1]:GetMaxHealth())
            end
            bg.Units = Data.Units
        end
    elseif Data.Type == 'Engineer' or Data.Type == 'Group' then
        local label = UIUtil.CreateText(bg, Data.Label, 10, UIUtil.bodyFont)
        LayoutHelpers.AtHorizontalCenterIn(label, icon)
        label.Bottom:Set(function() return bg.Bottom() - 4 end)
        label:SetDropShadow(true)
        label:DisableHitTest()
        
        local tabBG = Bitmap(bg, OIUtil.getImage('avatar_tab'))
        LayoutHelpers.AtLeftTopIn(tabBG, bg, -15, 0)
        tabBG:DisableHitTest()
        
        local tab = UIUtil.CreateText(tabBG, Data.Tab, 10, UIUtil.bodyFont)
        LayoutHelpers.AtCenterIn(tab, tabBG, 0, 0)
        tab:SetDropShadow(true)
        tab:DisableHitTest()
        
        bg.Update = function(Data)
            bg.Units = Data.Units
            label:SetText(Data.Label)
            tab:SetText(Data.Tab)
        end
    end
    
    return bg
end


function ToggleAvatars(state)
    showAvatars = state
    if UIUtil.GetAnimationPrefs() and avatarKey[1].icon and UIUtil.currentLayout != 'right' then
        avatarKey[1].icon:SetNeedsFrameUpdate(true)
        if state then
            animationLock = true
            avatarKey[1].icon.OnFrame = function(self, delta)
                if showAvatars then
                    local selfRight = self.Right()
                    if selfRight < avatarContainer.Right() then
                        animationLock = false
                        self.Right:Set(avatarContainer.Right)
                        self:SetNeedsFrameUpdate(false)
                    else
                        self.Right:Set(function() return selfRight - (delta * 500) end)
                    end
                end
            end
        else
            avatarKey[1].icon.OnFrame = function(self, delta)
                if not showAvatars then
                	--()((()-- 20 instead of 10
                    local globalLeft = avatarKey[table.getsize(avatarKey)].icon.Left() - 20
                    local selfRight = self.Right()
                    if globalLeft < avatarContainer.Right() then
                        self.Right:Set(function() return selfRight + (delta * 500) end)
                    else
                        self:SetNeedsFrameUpdate(false)
                    end
                end
            end
        end
    else
        for i, v in avatarKey do
            if v.icon then
                if showAvatars then
                    v.icon:Show()
                else
                    v.icon:Hide()
                end
            end
        end
    end
end


function applySkin()
	avatarToggle:SetNewTextures(
    	OIUtil.getImage('toggleavatar_on'),
        OIUtil.getImage('toggleavatar_off'),
        OIUtil.getImage('toggleavatar_on_mo'),
        OIUtil.getImage('toggleavatar_off_mo'),
        OIUtil.getImage('blank'),
        OIUtil.getImage('blank')
    )
end
