
local OIUtil = import('/mods/OllosInterface/libs/oiutil.lua')

function CommonLogic()
    controls.massLabel.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mass_button", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    controls.massWarning.OnFrame = function(self, deltaTime)
        massFlashing = true
        curAlpha = controls.massWarning:GetAlpha()
        --LOG('OI: ' .. curAlpha .. ' | ' .. deltaTime)
        if massAscending then
            curAlpha = curAlpha + (deltaTime * massMod)
        else
            curAlpha = curAlpha - (deltaTime * massMod)
        end
        if curAlpha > 1 then
            massCounter = massCounter + 1
            massAscending = false
            curAlpha = 1
        end
        if curAlpha < 0 then
            massAscending = true
            curAlpha = 0
        end
        controls.massWarning:SetAlpha(curAlpha)
        controls.massLabel:SetAlpha(1 - (0.5 * curAlpha))
        controls.massStatus._bar:SetAlpha(1 - (0.5 * curAlpha))
    end
    
    controls.energyLabel.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "energy_button", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    controls.energyWarning.OnFrame = function(self, deltaTime)
        energyFlashing = true
        curAlpha = controls.energyWarning:GetAlpha()
        --LOG('OI: ' .. curAlpha .. ' | ' .. deltaTime)
        if energyAscending then
            curAlpha = curAlpha + (deltaTime * energyMod)
        else
            curAlpha = curAlpha - (deltaTime * energyMod)
        end
        if curAlpha > 1 then
            energyCounter = energyCounter + 1
            energyAscending = false
            curAlpha = 1
        end
        if curAlpha < 0 then
            energyAscending = true
            curAlpha = 0
        end
        controls.energyWarning:SetAlpha(curAlpha)
        controls.energyLabel:SetAlpha(1 - (0.5 * curAlpha))
        controls.energyStatus._bar:SetAlpha(1 - (0.5 * curAlpha))
    end
    
    GameMain.AddBeatFunction(_BeatFunction)

    if controls.massDropOver then
        controls.massDropOver:DisableHitTest()
        controls.massDropOver:Hide()
    end
    
    controls.group.OnDestroy = function(self)
        GameMain.RemoveBeatFunction(_BeatFunction)
    end
    
    local massDropdownVisible = Prefs.GetFromCurrentProfile("massDetailedView") or true
    local energyDropdownVisisble = Prefs.GetFromCurrentProfile("energyDetailedView") or true
    
    if controls.energyDropdown then
        controls.energyDropdown:Hide()
        if energyDropdownVisisble then
            controls.energyDropdown:Show()
        end
    end
    
    if controls.massDropdown then
        controls.massDropdown:Hide()
        
        if massDropdownVisible then
            controls.massDropdown:Show()
        end
    end
    
    controls.massStorage.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(controls.massStorage, "storage", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end
    
    controls.massStatus.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(controls.massStorage, "storage", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    controls.energyStorage.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(controls.energyStorage, "storage", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end
    
    controls.energyStatus.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(controls.energyStorage, "storage", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end
    
    if controls.massRateContainer then
        Tooltip.AddButtonTooltip(controls.massRateContainer, "rate", .5)
        controls.massRateContainer.OnClick = function(self, modifiers)
            if massViewState >= 2 then
                massViewState = 1
            else
                massViewState = massViewState + 1
            end
            Prefs.SetToCurrentProfile("massRateView", massViewState)
        end
    end
    
    if controls.energyRateContainer then
        Tooltip.AddButtonTooltip(controls.energyRateContainer, "rate", .5)
        controls.energyRateContainer.OnClick = function(self, modifiers)
            if energyViewState >= 2 then
                energyViewState = 1
            else
                energyViewState = energyViewState + 1
            end
            Prefs.SetToCurrentProfile("energyRateView", energyViewState)
        end
    end
    
    if controls.massDropdownTarget then
        controls.massDropdownTarget.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(controls.massLabel, "extended_display", .5, true)
                if massDropdownVisible then
                    controls.massDropOver:Show()
                else
                    controls.massDropOver:Show()
                    controls.massDropdown:Show()
                end
                local sound = Sound({Bank = 'Interface', Cue = 'UI_Economy_Rollover'})
                PlaySound(sound)
            elseif event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
                if massDropdownVisible then
                    controls.massDropOver:Hide()
                else
                    controls.massDropOver:Hide()
                    controls.massDropdown:Hide()
                end
            elseif event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
                massDropdownVisible = not massDropdownVisible
                controls.massDropdown:SetHidden(not massDropdownVisible)
                controls.massDropOver:SetHidden(not massDropdownVisible)
                Prefs.SetToCurrentProfile("massDetailedView", massDropdownVisible)
                local sound = Sound({Bank = 'Interface', Cue = 'UI_Economy_Click'})
                PlaySound(sound)
            end
        end
    end

    if controls.energyDropdownTarget then
        controls.energyDropdownTarget.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(controls.energyLabel, "extended_display", .5, true)
                if energyDropdownVisisble then
                    controls.energyDropOver:Show()
                else
                    controls.energyDropOver:Show()
                    controls.energyDropdown:Show()
                end
                local sound = Sound({Bank = 'Interface', Cue = 'UI_Economy_Rollover'})
                PlaySound(sound)
            elseif event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
                if energyDropdownVisisble then
                    controls.energyDropOver:Hide()
                else
                    controls.energyDropOver:Hide()
                    controls.energyDropdown:Hide()
                end
            elseif event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
                energyDropdownVisisble = not energyDropdownVisisble
                controls.energyDropdown:SetHidden(not energyDropdownVisisble)
                controls.energyDropOver:SetHidden(not energyDropdownVisisble)
                Prefs.SetToCurrentProfile("energyDetailedView", energyDropdownVisisble)
                local sound = Sound({Bank = 'Interface', Cue = 'UI_Economy_Click'})
                PlaySound(sound)
            end
        end
    end

    return controls.massBG, controls.energyBG
end

function _BeatFunction()
    local econData = GetEconomyTotals()
    local simFrequency = GetSimTicksPerSecond()

    local function FormatRateString(RateVal, StoredVal, IncomeAvg, ActualAvg, RequestedAvg)
        local retRateStr = ''
        local retEffVal = 0
        if RateVal > 9999 then
            retRateStr = "+"..tostring(math.floor(RateVal / 1000)).."k"
        elseif RateVal >= 0 then
            retRateStr = "+"..tostring(RateVal)
        elseif RateVal < -9999 then
            retRateStr = tostring(math.floor(RateVal / 1000)).."k"
        elseif RateVal < 0 then
            retRateStr = tostring(RateVal)
        end
        if RequestedAvg == 0 then
            retEffVal = math.ceil(IncomeAvg) * 100
        else
            if StoredVal > 0.5 then
                retEffVal = math.ceil( (IncomeAvg / ActualAvg) * 100 )
            else
                retEffVal = math.ceil( (IncomeAvg / RequestedAvg) * 100 )
            end    
        end
        return retRateStr, retEffVal
    end

-- ----
-- MASS
-- ----        
    local massMaxStorageVal = econData["maxStorage"]["MASS"]
    local massStoredVal = econData["stored"]["MASS"]
    local massIncomeVal = econData["income"]["MASS"]
    local massLastRequestedVal = econData["lastUseRequested"]["MASS"]
    local massLastActualVal = econData["lastUseActual"]["MASS"]

    local massRequestedAvg = math.min(massLastRequestedVal * simFrequency, 99999)
    local massActualAvg = math.min(massLastActualVal * simFrequency, 99999)
    local massIncomeAvg = math.min(massIncomeVal * simFrequency, 99999)
    
    controls.massStatus:SetRange(0, massMaxStorageVal)
    controls.massStatus:SetValue(massStoredVal)
    controls.massStorage:SetText(string.format("%d / %d", massStoredVal, massMaxStorageVal))
    
    controls.massIncome:SetText(string.format("+%d", math.floor( massIncomeAvg + 0.5 ) ))
    if massStoredVal > 0.5 then
        controls.massExpense:SetText(string.format("-%d", math.floor( massActualAvg + 0.5 ) ))
    else
        controls.massExpense:SetText(string.format("-%d", math.floor( massRequestedAvg + 0.5 ) ))
    end

    local massRateVal = 0
    if massStoredVal > 0.5 then
        massRateVal = math.floor(( massIncomeAvg - massActualAvg ) + 0.5 )
    else
        massRateVal = math.floor(( massIncomeAvg - massRequestedAvg ) + 0.5 )
    end
    
    local massRateStr, massEffVal = FormatRateString(massRateVal, massStoredVal, massIncomeAvg, massActualAvg, massRequestedAvg)
-- CHOOSE RATE or EFFICIENCY STRING
    if massViewState == 1 then
        controls.massRate:SetText(string.format("%+s", massRateStr))
    elseif massViewState == 2 then
        controls.massRate:SetText(string.format("%d%%", math.min(massEffVal, 100)))   
    end
-- SET RATE/EFFICIENCY COLOR
    local massRateColor
    if massRateVal < 0 then
        if massStoredVal > 0 then
            massRateColor = OIUtil.getColor('economy_rate_yellow')
        else
            massRateColor = OIUtil.getColor('economy_rate_red')
        end
    else
        massRateColor = OIUtil.getColor('economy_rate_green')
    end
    controls.massRate:SetColor(massRateColor)

-- ------
-- ENERGY
-- ------        
    local energyMaxStorageVal = econData["maxStorage"]["ENERGY"]
    local energyStoredVal = econData["stored"]["ENERGY"]
    local energyIncomeVal = econData["income"]["ENERGY"]
    local energyLastRequestedVal = econData["lastUseRequested"]["ENERGY"]
    local energyLastActualVal = econData["lastUseActual"]["ENERGY"]

    local energyRequestedAvg = math.min(energyLastRequestedVal * simFrequency, 99999)
    local energyActualAvg = math.min(energyLastActualVal * simFrequency, 99999)
    local energyIncomeAvg = math.min(energyIncomeVal * simFrequency, 99999)
    
    controls.energyStatus:SetRange(0, energyMaxStorageVal)
    controls.energyStatus:SetValue(energyStoredVal)
    controls.energyStorage:SetText(string.format("%d / %d", energyStoredVal, energyMaxStorageVal))

    controls.energyIncome:SetText(string.format("+%d", math.floor( energyIncomeAvg + 0.5 ) ))
    if energyStoredVal > 0.5 then
        controls.energyExpense:SetText(string.format("-%d", math.floor( energyActualAvg + 0.5 ) ))
    else
        controls.energyExpense:SetText(string.format("-%d", math.floor( energyRequestedAvg + 0.5 ) ))
    end

-- GENERATE RATE STRING
    local energyRateVal = 0
    if energyStoredVal > 0.5 then
        energyRateVal = math.floor(( energyIncomeAvg - energyActualAvg ) + 0.5 )
    else
        energyRateVal = math.floor(( energyIncomeAvg - energyRequestedAvg ) + 0.5 )
    end
    local energyRateStr, energyEffVal = FormatRateString(energyRateVal, energyStoredVal, energyIncomeAvg, energyActualAvg, energyRequestedAvg)
-- CHOOSE RATE or EFFICIENCY STRING
    if energyViewState == 1 then
        controls.energyRate:SetText(string.format("%s", energyRateStr))
    elseif energyViewState == 2 then
        controls.energyRate:SetText(string.format("%d%%", math.min(energyEffVal, 100)))
    end
-- SET RATE/EFFICIENCY COLOR
    local energyRateColor
    if energyRateVal < 0 then
        if energyStoredVal > 0 then
            energyRateColor = OIUtil.getColor('economy_rate_yellow')
        else
            energyRateColor = OIUtil.getColor('economy_rate_red')
        end
    else
        energyRateColor = OIUtil.getColor('economy_rate_green')
    end
    controls.energyRate:SetColor(energyRateColor)

-- ECONOMY WARNINGS        
    if Prefs.GetOption('econ_warnings') and not hideWarnings then
        if UIState then
        	
            if massStoredVal / massMaxStorageVal < .2 then
                if not massFlashing then
                    if massEffVal < 25 then
                        controls.massWarning:SetNeedsFrameUpdate(true)
                        massFlashing = true
                        controls.massWarning:Show()
                        massMod = 2
                    elseif massEffVal < 75 then
                        controls.massWarning:SetNeedsFrameUpdate(true)
                        massFlashing = true
                        controls.massWarning:Show()
                        massMod = 1
                    elseif massEffVal <= 100 then
                        controls.massWarning:SetNeedsFrameUpdate(true)
                        massFlashing = true
                        controls.massWarning:Show()
                        massMod = .5
                    elseif massEffVal > 100 then
                        controls.massWarning:SetNeedsFrameUpdate(false)
                        massFlashing = false
                        controls.massWarning:Hide()
                    end
                end
            else
            	controls.massLabel:SetNeedsFrameUpdate(false)
                controls.massLabel:SetAlpha(1)
                controls.massStatus._bar:SetAlpha(1)
                controls.massWarning:SetNeedsFrameUpdate(false)
                controls.massWarning:Hide()
                massFlashing = false
            end
            
            if energyStoredVal / energyMaxStorageVal < .2 then
                if not energyFlashing then
                    if energyEffVal < 25 then
                        controls.energyWarning:SetNeedsFrameUpdate(true)
                        energyFlashing = true
                        controls.energyWarning:Show()
                        energyMod = 2
                    elseif energyEffVal < 75 then
                        controls.energyWarning:SetNeedsFrameUpdate(true)
                        energyFlashing = true
                        energyMod = 1
                        controls.energyWarning:Show()
                    elseif energyEffVal <= 100 then
                        controls.energyWarning:SetNeedsFrameUpdate(true)
                        energyFlashing = true
                        energyMod = .5
                        controls.energyWarning:Show()
                    elseif energyEffVal > 100 then
                        energyFlashing = false
                        controls.energyWarning:SetNeedsFrameUpdate(false)
                        controls.energyWarning:Hide()
                    end
                end
            else
            	controls.energyLabel:SetNeedsFrameUpdate(false)
                controls.energyLabel:SetAlpha(1)
                controls.energyStatus._bar:SetAlpha(1)
                controls.energyWarning:SetNeedsFrameUpdate(false)
                controls.energyWarning:Hide()
                energyFlashing = false
            end
        else
            controls.massWarning:Hide()
            controls.energyWarning:Hide()
        end
    else
        controls.massWarning:Hide()
        controls.energyWarning:Hide()
    end
end