
local OIUtil = import('/mods/OllosInterface/libs/oiutil.lua')

local function CreateMFDCheckbox(name, behavior, label, isbutton)
    local centeringGroup = Group(controls.bg, "centeringGroup")
    centeringGroup.Width:Set(44)
    --()((()-- 29 instead of 33
    centeringGroup.Height:Set(29)
    if not isbutton then
    	--()((() replace textures
        centeringGroup.button = UIUtil.CreateCheckbox(centeringGroup,
            '/mods/OllosInterface/textures/icons_mfd/' .. name .. "_btn_up.dds",
	        '/mods/OllosInterface/textures/icons_mfd/' .. name .. "_btn_down.dds",
	        '/mods/OllosInterface/textures/icons_mfd/' .. name .. "_btn_over.dds",
	        '/mods/OllosInterface/textures/icons_mfd/' .. name .. "_btn_over.dds",
	        '/mods/OllosInterface/textures/icons_mfd/' .. name .. "_btn_dis.dds",
	        '/mods/OllosInterface/textures/icons_mfd/' .. name .. "_btn_dis.dds",
            "UI_MFD_Click", "UI_MFD_Rollover")
        --()((()--
        centeringGroup.button.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(self, label, nil, true)
            elseif event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
            end
            Checkbox.HandleEvent(self, event)
        end
        centeringGroup.button.OnCheck = behavior
    else
    	--()((()-- replace textures
        centeringGroup.button = OIUtil.CreateButtonFullPath(centeringGroup,
        	'/mods/OllosInterface/textures/icons_mfd/' .. name .. "_btn_up.dds",
	        '/mods/OllosInterface/textures/icons_mfd/' .. name .. "_btn_down.dds",
	        '/mods/OllosInterface/textures/icons_mfd/' .. name .. "_btn_over.dds",
	        '/mods/OllosInterface/textures/icons_mfd/' .. name .. "_btn_dis.dds",
            nil, nil, nil, nil, "UI_MFD_Click", "UI_MFD_Rollover")
        --()((()--		
        centeringGroup.button.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(self, label, nil, true)
            elseif event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
            end
            Button.HandleEvent(self, event)
        end
        centeringGroup.button.OnClick = behavior
    end	
	
    LayoutHelpers.AtCenterIn(centeringGroup.button, centeringGroup)
    return centeringGroup
end

function SetLayout(minimap, onToggleMinimap)
    if not onToggleMinimap then
        if controls.displayGroup then controls.displayGroup:Destroy() controls.displayGroup = false end
        if controls.miniMap then controls.miniMap:Destroy() controls.miniMap = false end
        if controls.glow then controls.glow = false end
    end
    if not controls.group then
        controls.group = Group(controls.controlClusterGroup, "MFD Group")
    
        controls.group:DisableHitTest()
    end

    if not controls.bg then
        controls.bg = Bitmap(controls.group)
    end

    for i = 1, table.getn(buttonData) do
        if not controls.minimapBtns[i] then
            controls.minimapBtns[i] = CreateMFDCheckbox(buttonData[i].btnName, buttonData[i].clickBehavior, buttonData[i].label, buttonData[i].button)
            controls.minimapBtns[i].Depth:Set(function() return controls.bg.Depth() + 1 end)
        end
    end
    
    
    if not controls.displayGroup then
        controls.displayGroup = Group(controls.controlClusterGroup,"MFD Display Group")
        controls.displayGroup.Depth:Set(3)
    end
    if not controls.miniMap then
        controls.miniMap = import('/lua/ui/controls/worldview.lua').WorldView(controls.displayGroup, 'MiniMap', 2, true, 'WorldCamera')    -- depth value is above minimap
        controls.miniMap:SetName("Minimap")
        controls.miniMap:Register('MiniMap', true, '<LOC map_view_0000>MiniMap')
        controls.miniMap:SetRenderPass(UIUtil.UIRP_UnderWorld) -- don't change this or the camera will lag one frame behind
        controls.miniMap.Depth:Set(10)
        controls.miniMap:SetNeedsFrameUpdate(true)
        -- defer the camera reset one frame so a view is created, this causes a fully zoomed out map
        local frameCount = 0
        controls.miniMap.OnFrame = function(self, elapsedTime)
            if frameCount == 1 then
                controls.miniMap:CameraReset()
                controls.miniMap.OnFrame = nil  -- we want the control to continue to get frame updates in the engine, but not in Lua. PLEASE DON'T CHANGE THIS OR IT BREAKS CAMERA DRAGGING
            end
            frameCount = frameCount + 1
        end
        LayoutHelpers.FillParent(controls.miniMap, controls.displayGroup)
        if not minimap then 
            controls.displayGroup:Hide()
        end 
    end
    if not controls.miniMap.BG then
        controls.miniMap.BG = Bitmap(controls.miniMap)
        controls.miniMap.BG.Depth:Set(function() return controls.miniMap.Depth() - 1 end) 
    end

    if not controls.glow then
        controls.glow = Bitmap(controls.miniMap.BG)
        
        --()((()-- use other texture and additional 'glow'
        controls.glow:SetTexture(OIUtil.getImage('minimap_h_fg'), 2)
        controls.glow:DisableHitTest()
        controls.glow.Depth:Set(function() return controls.miniMap.Depth() + 2 end)
        controls.glow:SetRenderPass(UIUtil.UIRP_PostGlow)
        LayoutHelpers.AtCenterIn(controls.glow, controls.miniMap)
         
        controls.glow2 = Bitmap(controls.miniMap.BG)
        controls.glow2:SetTexture(OIUtil.getImage('minimap_glow'), 2)
        controls.glow2:DisableHitTest()
        controls.glow2.Depth:Set(function() return controls.glow.Depth() + 1 end)
        controls.glow2:SetRenderPass(UIUtil.UIRP_PostGlow)
        LayoutHelpers.AtCenterIn(controls.glow2, controls.miniMap)
        --()((()--
        
    end
    
    for i, v in controls.minimapBtns do
        local tempLayoutName = "l_mfd_left_btn_" .. tostring(i)
        --()((()-- use different positioning
        LayoutHelpers.AtLeftTopIn(controls.minimapBtns[i], controls.bg, 8, 11 + 38 * (i-1)) 
        --[[
        LayoutHelpers.RelativeTo(controls.minimapBtns[i],
            controls.group,
            UIUtil.SkinnableFile('/game/mfd_bmp/mfd_bmp_layout.lua'),
            tempLayoutName,
            "mfd_button_bmp",
            0, 2)
        --]]
        --()((()--
    end
    
    --()((()-- other positioning and stuff
    --controls.group.Top:Set(function() return controls.controlClusterGroup.Top() end)
    controls.group.Top:Set(function() return controls.bg.Top() end)
    controls.group.Left:Set(function() return controls.controlClusterGroup.Left() - 2 end)
    controls.group.Bottom:Set(controls.controlClusterGroup.Bottom)
    LayoutHelpers.ResetRight(controls.group)
    
    --controls.bg:SetTexture(UIUtil.UIFile('/game/mfd_bmp/mfd_button_bmp.dds'))
    controls.bg:SetTexture(OIUtil.getImage('overlays_h_bg'))
    controls.bg.Left:Set(controls.group.Left)
    --controls.bg.Top:Set(controls.group.Top)
    controls.bg.Bottom:Set(controls.group.Bottom)
    
    --controls.miniMap.BG:SetTexture(UIUtil.UIFile('/game/mfd_bmp/mfd_bmp.dds'))
    controls.miniMap.BG:SetTexture(OIUtil.getImage('minimap_h_bg'), 2)
    controls.miniMap.BG.Left:Set(function() return controls.bg.Right() end)
    --controls.miniMap.BG.Top:Set(controls.bg.Top)
    controls.miniMap.BG.Bottom:Set(controls.bg.Bottom)
    
    LayoutHelpers.AtCenterIn(controls.displayGroup, controls.miniMap.BG)
    controls.displayGroup.Height:Set(180)
    controls.displayGroup.Width:Set(180)
    
    -- shadows
    if not controls.shadowTop then
	    controls.shadowTop = Bitmap(controls.group, OIUtil.getImage('shadow_top'))
	    controls.shadowTop.Bottom:Set(function() return controls.group.Top() end)
	    controls.shadowTop.Left:Set(function() return controls.group.Left() end)
		controls.shadowTop.Right:Set(function() return controls.group.Right() end)
	end
	--()((()
    
    --[[
    controls.group.Top:Set(function() return controls.controlClusterGroup.Top() end)
    controls.group.Left:Set(function() return controls.controlClusterGroup.Left() - 2 end)
    controls.group.Bottom:Set(controls.controlClusterGroup.Bottom)
    LayoutHelpers.ResetRight(controls.group)
    
    controls.bg:SetTexture(UIUtil.UIFile('/game/mfd_bmp/mfd_button_bmp.dds'))
    controls.bg.Left:Set(controls.group.Left)
    controls.bg.Top:Set(controls.group.Top)
    
    controls.miniMap.BG:SetTexture(UIUtil.UIFile('/game/mfd_bmp/mfd_bmp.dds'))
    controls.miniMap.BG.Left:Set(function() return controls.bg.Right() - 4 end)
    controls.miniMap.BG.Top:Set(controls.bg.Top)
    
    LayoutHelpers.AtCenterIn(controls.displayGroup, controls.miniMap.BG)
    controls.displayGroup.Height:Set(180)
    controls.displayGroup.Width:Set(180)
	--]]
	
    if minimap then
        controls.group.Width:Set(function() return controls.miniMap.BG.Width() + controls.bg.Width() end)
    else
        controls.group.Width:Set(function() return controls.bg.Width() end)
    end
    LayoutHelpers.ResetRight(controls.group)
    LayoutHelpers.ResetHeight(controls.group)
end



    