
local OIUtil = import('/mods/OllosInterface/libs/oiutil.lua')
local controls_const = import('/lua/ui/game/construction.lua').controls



function CreateResourceGroup(parent, groupLabel)
    local group = Group(parent)
    
    # Group label
    group.Label = UIUtil.CreateText( group, groupLabel, 12, "Arial Bold" )
    group.Label:SetColor("FFBEBEBE")

    # Energy Icon
    group.EnergyIcon = Bitmap(group)
    group.EnergyIcon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-energy_bmp.dds'))

    # Energy Value
    group.EnergyValue = UIUtil.CreateText( group, "0", 12, UIUtil.bodyFont )
    group.EnergyValue:SetColor("FF00F000")

    # Mass Icon
    group.MassIcon = Bitmap(group)
    group.MassIcon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-mass_bmp.dds'))

    # Mass Value
    group.MassValue = UIUtil.CreateText( group, "0", 12, UIUtil.bodyFont )
    group.MassValue:SetColor("FF00F000")

    return group
end

# A 'Stat Group' is an icon or text label with a value on the right
# e.g.  Health 3000
#       Shield 6000
#       <icon> 24
function CreateStatGroup(parent, labelIcon)
    local group = Group(parent)
    
    group.Label = Bitmap(group)
    group.Label:SetTexture(labelIcon)

    group.Value = UIUtil.CreateText( group, "", 14, UIUtil.bodyFont )
    group.Value:SetColor("FF00F000")

    return group
end

function CreateTextbox(parent, label, bigBG)
    local group = Group(parent)
    
    if bigBG then
        group.BGTop = Bitmap(group)
        group.BGMid = Bitmap(group)
        group.BGBot = Bitmap(group)
        group.BGTop:SetTexture(OIUtil.getImage('text_fulltop'))
        group.BGMid:SetTexture(OIUtil.getImage('text_fullmiddle'))
        group.BGBot:SetTexture(OIUtil.getImage('text_fullbottom'))
    else
        group.bgtopleft = Bitmap(group)
        group.bgtopmiddle = Bitmap(group)
        group.bgtopright = Bitmap(group)
        
        group.bgmidleft = Bitmap(group)
        group.bgmidmiddle = Bitmap(group)
        group.bgmidright = Bitmap(group)
        
        group.bgbotleft = Bitmap(group)
        group.bgbotmiddle = Bitmap(group)
        group.bgbotright = Bitmap(group)
        
        group.bgtopleft:SetTexture(OIUtil.getImage('text_topleft'))
        group.bgtopmiddle:SetTexture(OIUtil.getImage('text_top'))
        group.bgtopright:SetTexture(OIUtil.getImage('text_topright'))
        
        group.bgmidleft:SetTexture(OIUtil.getImage('text_left'))
        group.bgmidmiddle:SetTexture(OIUtil.getImage('text_bg'))
        group.bgmidright:SetTexture(OIUtil.getImage('text_right'))
        
        group.bgbotleft:SetTexture(OIUtil.getImage('text_bottomleft'))
        group.bgbotmiddle:SetTexture(OIUtil.getImage('text_bottom'))
        group.bgbotright:SetTexture(OIUtil.getImage('text_bottomright'))
    end
    
    if label then
        group.Label = UIUtil.CreateText( group, LOC(label), 12, "Arial Bold" )
        group.Label:SetColor("FFBEBEBE")
    end

    group.Value = {}
    group.Value[1] = UIUtil.CreateText( group, "", 12, UIUtil.bodyFont)
    
    return group
end

function Create(parent)
    # Main window
    if not import('/lua/ui/game/unitviewDetail.lua').View then
        import('/lua/ui/game/unitviewDetail.lua').View = Group(parent)
    end
    
    local View = import('/lua/ui/game/unitviewDetail.lua').View
    
    if not View.BG then
        View.BG = Bitmap(View)
    end
    --View.BG:SetTexture(UIUtil.UIFile('/game/build-ui-02/build-over_bmp.dds'))
    View.BG:SetTexture(OIUtil.getImage('unitdetails_bg'))
    View.BG.Depth:Set(200)

    # Unit icon
    if false then
        View.UnitIcon = Bitmap(View)
    end
    
    if not View.UnitImg then
        View.UnitImg = Bitmap( View.BG )
    end
    
    # Unit Name
    if not View.UnitName then
        View.UnitName = UIUtil.CreateText( View.BG, "", 12, "Arial Bold" )
    end
    View.UnitName:SetColor("FFFF9E06")

    # Unit Description
    if not View.UnitShortDesc then
        View.UnitShortDesc = UIUtil.CreateText( View.BG, "", 12, UIUtil.bodyFont )
    end
    View.UnitShortDesc:SetColor("FFFF9E06")

    # Cost
    if not View.BuildCostGroup then
        View.BuildCostGroup = CreateResourceGroup(View.BG, "<LOC uvd_0000>Build Cost (Rate)")
    end

    # Upkeep
    if not View.UpkeepGroup then
        View.UpkeepGroup = CreateResourceGroup(View.BG, "<LOC uvd_0002>Yield")
    end
    
    # Health stat
    if not View.HealthStat then
        View.HealthStat = CreateStatGroup( View.BG, UIUtil.UIFile('/game/unit_view_icons/redcross.dds') )
    end
    
    if not View.ShieldStat then
        View.ShieldStat = CreateStatGroup( View.BG, UIUtil.UIFile('/game/unit_view_icons/shield.dds') )
    end
    # Tme stat
    if not View.TimeStat then
        View.TimeStat = CreateStatGroup( View.BG, UIUtil.UIFile('/game/unit-over/icon-clock_bmp.dds') )
    end
    
    if Prefs.GetOption('uvd_format') == 'full' then
        # Description  "<LOC uvd_0003>Description"
        if not View.Description then
            View.Description = CreateTextbox(View.BG, nil, true)
            --View.Description = CreateTextbox(View.BG, nil)
        end
        
        # Ability List"<LOC _Abilities>"
        if not View.AbilityList then
            View.AbilityList = CreateTextbox(View.BG, nil)
        end
    else
        if View.Description then View.Description:Destroy() View.Description = false end
        if View.AbilityList then View.AbilityList:Destroy() View.AbilityList = false end
    end
    
    View.BG:DisableHitTest(true)
end

function SetLayout()
    import('/lua/ui/game/unitviewDetail.lua').ViewState = Prefs.GetOption('uvd_format')
    local mapGroup = import('/lua/ui/game/unitviewDetail.lua').MapView
    
    Create(mapGroup)
    
    local control = import('/lua/ui/game/unitviewDetail.lua').View

    local OrderGroup = import('/lua/ui/game/orders.lua').controls.bg

    --LayoutHelpers.AtLeftIn(control, mapGroup, 0)
    --LayoutHelpers.AtBottomIn(control, mapGroup, -2)
    control.Width:Set( control.BG.Width )
    control.Height:Set( control.BG.Height )
    
    -- position aligned to construction controls
    control.Bottom:Set(function() return mapGroup.Bottom() + 2 end)
    control.Left:Set(function() return controls_const.unitsBGTop.Right() - 1 end)
    
    # Main window background
    LayoutHelpers.AtLeftTopIn( control.BG, control )

    # Unit Name
    LayoutHelpers.AtLeftTopIn( control.UnitName, control.BG, 10, 8)
    
    # Unit Image
    LayoutHelpers.AtLeftTopIn( control.UnitImg, control.BG, 7, 31 )
    control.UnitImg.Height:Set(46)
    control.UnitImg.Width:Set(48)
    
    # Unit Description
    control.UnitShortDesc.Left:Set(function() return control.UnitName.Right() + 5 end)
    control.UnitShortDesc.Bottom:Set(control.UnitName.Bottom)

    # Health Stat
    LayoutHelpers.AtLeftTopIn(control.HealthStat, control.BG, 64, 31)
    control.HealthStat.Width:Set(1)
    control.HealthStat.Height:Set(control.HealthStat.Label.Height)
    LayoutStatGroup( control.HealthStat )
    
    # Shield stat
    LayoutHelpers.Below( control.TimeStat, control.HealthStat, 2 )
    control.TimeStat.Width:Set(1)
    control.TimeStat.Height:Set(control.TimeStat.Label.Height)
    LayoutStatGroup( control.TimeStat )
    
    LayoutHelpers.Below(  control.ShieldStat, control.TimeStat, 2 )
    control.TimeStat.Width:Set(1)
    control.TimeStat.Height:Set(control.ShieldStat.Label.Height)
    LayoutStatGroup( control.ShieldStat )
    
    # Build Resource Group
    LayoutHelpers.AtLeftTopIn( control.BuildCostGroup, control.BG, 152, 28 )
    control.BuildCostGroup.Width:Set( 120 )
    LayoutResourceGroup( control.BuildCostGroup )
    control.BuildCostGroup.Bottom:Set( function() return control.BuildCostGroup.MassValue.Bottom() + 1 end )

    # Upkeep Resource Group
    LayoutHelpers.RightOf( control.UpkeepGroup, control.BuildCostGroup )
    control.UpkeepGroup.Width:Set( 80 )
    control.UpkeepGroup.Bottom:Set( control.BuildCostGroup.Bottom )
    LayoutResourceGroup( control.UpkeepGroup )
    
    if control.AbilityList then
        # Ability List
        control.AbilityList.Left:Set(function() return control.BG.Left() + 15 end)
        control.AbilityList.Bottom:Set(function() return control.BG.Top() - 13 end)
        control.AbilityList.Width:Set(20)
        LayoutAbilityBox(control.AbilityList)
   	end
        
   	if control.Description then
        # Description
        control.Description.Left:Set(function() return control.BG.Right() - 4 end)
        control.Description.Top:Set(function() return control.BG.Top() + 1 end)
        control.Description.Bottom:Set(function() return control.BG.Bottom() - 2 end)
        control.Description.Width:Set(control.Description.BGTop.Width)
        LayoutTextbox(control.Description)
        
   	end    
    
    control:Hide()
end

function LayoutResourceGroup(group)
    
    LayoutHelpers.AtTopIn( group.Label, group )
    LayoutHelpers.AtHorizontalCenterIn( group.Label, group )

    LayoutHelpers.Below( group.EnergyIcon, group.Label )
    group.EnergyIcon.Left:Set( function() return group.Label.Left() - 4 end )

    LayoutHelpers.RightOf( group.EnergyValue, group.EnergyIcon, 1 )
    group.EnergyValue.Top:Set( function() return group.EnergyIcon.Top() + 1 end )

    LayoutHelpers.RightOf( group.MassValue, group.MassIcon, 1 )
    group.MassValue.Right:Set( function() return group.Label.Right() end )
    group.MassValue.Top:Set( function() return group.MassIcon.Top() + 1 end )

    LayoutHelpers.Below( group.MassIcon, group.EnergyIcon )
end

function LayoutStatGroup(group)
    group.Label.Left:Set(group.Left)
    group.Label.Top:Set(group.Top)
    group.Value.Left:Set(function() return group.Label.Right() + 4 end)
    LayoutHelpers.AtVerticalCenterIn(group.Value, group.Label)
end  

function LayoutTextbox(group)
    group.BGBot.Bottom:Set(group.Bottom)
    group.BGTop.Top:Set(group.Top)
    group.BGMid.Bottom:Set(group.BGBot.Top)
    group.BGMid.Top:Set(group.BGTop.Bottom)
    
    group.BGBot.Left:Set(group.Left)
    group.BGTop.Left:Set(group.Left)
    group.BGMid.Left:Set(group.Left)
    
    
    
    group.BGTop.Depth:Set(function() return group.Depth() - 1 end)
    group.BGMid.Depth:Set(group.BGTop.Depth)
    group.BGBot.Depth:Set(group.BGTop.Depth)
    
    if group.Label then
        LayoutHelpers.AtRightTopIn(group.Label, group.BGTop, 10, 5)
        group.Label:SetColor("FFFF9E06")
    end
    
    LayoutHelpers.AtLeftTopIn(group.Value[1], group.BGTop, 8, 8)
    group.Value[1].Right:Set(function() return group.Right() - 15 end)
    group.Value[1].Width:Set(function() return group.Right() - group.Left() - 14 end)
    group.Value[1]:SetClipToWidth(true)
end

function LayoutAbilityBox(group)
    group.bgmidmiddle.Left:Set(group.Left)
    group.bgmidmiddle.Bottom:Set(group.Bottom)
    
    group.bgtopleft.Right:Set(group.bgmidmiddle.Left)
    group.bgtopleft.Bottom:Set(group.bgmidmiddle.Top)
    group.bgtopright.Left:Set(group.bgmidmiddle.Right)
    group.bgtopright.Bottom:Set(group.bgmidmiddle.Top)
    
    group.bgbotleft.Right:Set(group.bgmidmiddle.Left)
    group.bgbotleft.Top:Set(group.bgmidmiddle.Bottom)
    group.bgbotright.Left:Set(group.bgmidmiddle.Right)
    group.bgbotright.Top:Set(group.bgmidmiddle.Bottom)
     
    group.bgtopmiddle.Bottom:Set(group.bgmidmiddle.Top)
    group.bgtopmiddle.Left:Set(group.bgmidmiddle.Left)
    group.bgtopmiddle.Right:Set(group.bgmidmiddle.Right)
    group.bgbotmiddle.Top:Set(group.bgmidmiddle.Bottom)
    group.bgbotmiddle.Left:Set(group.bgmidmiddle.Left)
    group.bgbotmiddle.Right:Set(group.bgmidmiddle.Right)
    
    group.bgmidleft.Bottom:Set(group.bgmidmiddle.Bottom)
    group.bgmidleft.Top:Set(group.bgmidmiddle.Top)
    group.bgmidleft.Right:Set(group.bgmidmiddle.Left)
    group.bgmidright.Bottom:Set(group.bgmidmiddle.Bottom)
    group.bgmidright.Top:Set(group.bgmidmiddle.Top)
    group.bgmidright.Left:Set(group.bgmidmiddle.Right)
        
    LayoutHelpers.AtLeftTopIn(group.Value[1], group.bgmidmiddle, -5, -5)
end