--*****************************************************************************
--* File: lua/modules/ui/game/queue.lua
--* Author: Chris Blackwell / Ted Snook
--* Summary: Construction Queue UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local OIUtil = import('/mods/OllosInterface/libs/oiutil.lua')
local Prefs = import('/lua/user/prefs.lua')

function SetQueueGrid(queue, selection)
    currentQueue = queue
    
    local numIcons = table.getn(queue)

    PopulateQueueGrid(numIcons)

    for i = 1, numIcons do
        local icon = controls.queueGrid:GetItem(i, 1)
        if not icon then
            LOG("SetQueueGrid attempt to get item ", i, " but it's not there.")
            continue
        end
        local bpid = queue[i].id
        local blueprint = __blueprints[bpid]
        local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
        icon.unitIcon:SetTexture(upIconName)
        icon.text:SetText(tostring(queue[i].count))
        icon._blueprintId = bpid
        icon._index = i -- store array index for determining who was clicked on
		
		-- ()((() -- add strategic icon if possible
		OIUtil.addStrategicIconTo(icon.unitIcon, blueprint)
		
		-- ()((() -- added MouseWheel support
        icon.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                local blueprint = __blueprints[self._blueprintId]
                import('/lua/ui/game/unitviewDetail.lua').Show(blueprint, currentSelection[1], self._blueprintId)
            elseif event.Type == 'MouseExit' then
                    import('/lua/ui/game/unitviewDetail.lua').Hide()
            elseif event.Type == 'ButtonPress' 
            	or event.Type == 'ButtonDClick' 
                or event.Type == 'WheelRotation' then
                
                --[[
                if event.Type == 'WheelRotation' then
                	LOG(event.WheelRotation .. '|')
                end
                --]]
                
                -- ()((() -- use custom values
                local count = 1
                if event.Modifiers.Ctrl == true and event.Modifiers.Shift == nil then
                    count = Prefs.GetOption('oi_build_ctrlvalue')
                elseif event.Modifiers.Shift == true and event.Modifiers.Ctrl == nil then
                    count = Prefs.GetOption('oi_build_shiftvalue')
                elseif event.Modifiers.Shift == true and event.Modifiers.Ctrl == true then
                    count = Prefs.GetOption('oi_build_ctrlshiftvalue')
                end
                
                
                if event.Modifiers.Left or event.WheelRotation > 0 then
                    IncreaseBuildCountInQueue(self._index, count)
                else
                	local sValue = self.text:GetText()
                	local iValue = tonumber(sValue)
                	iValue = iValue - count
                	--LOG(iValue)
                	if iValue <= 0 then
                		ForkThread(OIUtil.updateStrategicIcons)
                	end
                    DecreaseBuildCountInQueue(self._index, count)
                end
            end
            -- ()((() --
            Bitmap.HandleEvent(self, event)
        end

    end
    controls.queueGrid:ScrollSetTop('Horz', 1)
    controls.queueGrid:EndBatch()
    local visCols, visRows = controls.queueGrid:GetVisible()
    if (visCols * visRows) >= numIcons then
        controls.scrollQueueFwdButton:Disable()
        controls.scrollQueueBackButton:Disable()
    else
        controls.scrollQueueFwdButton:Enable()
        controls.scrollQueueBackButton:Enable()
    end
end


