
local OIUtil = import('/mods/OllosInterface/libs/oiutil.lua')

function CreateArmyLine(parent)
    local group = Group(parent)
    
    group.faction = Bitmap(group)
    group.teamcolor = Bitmap(group)
    group.playername = UIUtil.CreateText(group, "", 12, UIUtil.bodyFont)    
    group.score = UIUtil.CreateText(group, "", 12, UIUtil.bodyFont)
    group.nameBG = Bitmap(group, OIUtil.getImage('blank'))
    group.scoreBG = Bitmap(group, OIUtil.getImage('blank'))
    
    group.middlebg = Bitmap(group, OIUtil.getImage('score_line_middle'))
    group.rightbg = Bitmap(group, OIUtil.getImage('score_line_right'))
    group.leftbg = Bitmap(group, OIUtil.getImage('score_line_left'))
    
    return group
end

function LayoutLine(line)

    line.faction.Left:Set(function() return line.Left() + 5 end)
    line.faction.Bottom:Set(function() return line.Bottom() - 1 end)
    line.faction.Height:Set(15)
    line.faction.Width:Set(15)
    LayoutHelpers.FillParent(line.teamcolor, line.faction)
    line.teamcolor.Depth:Set(function() return line.faction.Depth() - 1 end)
    
    line.nameBG.Left:Set(function() return line.faction.Right() + 7 end)
    line.nameBG.Width:Set(200)
    line.nameBG.Bottom:Set(line.Bottom)
    line.playername.Left:Set(line.nameBG.Left)
    
    LayoutHelpers.AtVerticalCenterIn(line.score, line)
    LayoutHelpers.AtVerticalCenterIn(line.playername, line)
    
    line.scoreBG.Left:Set(line.nameBG.Right)
    line.scoreBG.Width:Set(100)
    line.scoreBG.Bottom:Set(line.Bottom)
    line.score.Right:Set(line.scoreBG.Right)
    
    
    line.nameBG.Depth:Set(function() return line.playername.Depth() - 1 end)
    
    line.leftbg.Left:Set(function() return line.Left() end)
    line.leftbg.Bottom:Set(line.middlebg.Bottom)
    line.leftbg.Depth:Set(line.middlebg.Depth)
    
    line.rightbg.Right:Set(function() return line.scoreBG.Right() + 5 end)
    line.rightbg.Bottom:Set(line.middlebg.Bottom)
    line.rightbg.Depth:Set(line.middlebg.Depth)
    
    line.middlebg.Left:Set(line.leftbg.Right)
    line.middlebg.Right:Set(line.rightbg.Left)
    line.middlebg.Bottom:Set(line.Bottom)
    line.middlebg.Depth:Set(function() return line.nameBG.Depth() - 1 end)
    
    
end

function SetLayout()
	if true then
		return true
    end
    local worldView = import('/lua/ui/game/borders.lua').GetMapGroup()
    if UIUtil.currentLayout == 'minimal' then
        overlay.group.Top:Set(worldView.Top)
        overlay.group.Left:Set(worldView.Left)
        overlay.group.Right:Set(function() return worldView.Right() - 13 end)
        overlay.group.Bottom:Set(function() return worldView.Bottom() - 150 end)
    else
        overlay.group.Top:Set(worldView.Top)
        overlay.group.Left:Set(worldView.Left)
        overlay.group.Right:Set(function() return worldView.Right() - 13 end)
        overlay.group.Bottom:Set(function() return worldView.Bottom() - 33 end)
    end
end

function UpdateTime()
    while overlay.time do
        local time = os.date("*t")
        local ampm = ""
        if time.hour > 11 then
            ampm = "<LOC _PM>PM"
        else
            ampm = "<LOC _AM>AM"
        end
        overlay.time:SetText(LOCF("%02d:%02d:%02d %s", math.mod(time.hour, 12), time.min, time.sec, ampm))
        WaitSeconds(1)
    end
end

function CreateScoreOverlay()
    local worldView = import('/lua/ui/game/borders.lua').GetMapGroup()
    
    local armiesInfo = GetArmiesTable()
    
    overlay.group = Group(worldView, "scoreOverlayGroup")
    overlay.group:SetRenderPass(UIUtil.UIRP_PostGlow)
    LayoutHelpers.AtLeftTopIn(overlay.group, worldView, 0, 18)
    
    local counter = 1
    for i, v in armiesInfo.armiesTable do
        if v.showScore and not v.civilian then
            overlay.armyLines[counter] = CreateArmyLine(overlay.group)
            counter = counter + 1
        end
    end
    
    local index = 0
    for i, v in overlay.armyLines do
    	
        index = i
        if index == 1 then
            v.Top:Set(overlay.group.Top)
        else
            v.Top:Set(overlay.armyLines[index-1].Bottom)
        end
        LayoutLine(v)
        v.Left:Set(overlay.group.Left)
        v.Height:Set(v.middlebg.Height)
    end
    
    
    overlay.topLeft = Bitmap(overlay.group, OIUtil.getImage('score_border_topleft'))
    overlay.topRight = Bitmap(overlay.group, OIUtil.getImage('score_border_topright'))
    overlay.topMiddle = Bitmap(overlay.group, OIUtil.getImage('score_border_top'))
    overlay.bottomRight = Bitmap(overlay.group, OIUtil.getImage('score_border_bottomright'))
    overlay.bottomLeft = Bitmap(overlay.group, OIUtil.getImage('score_border_bottomleft'))
    overlay.bottomMiddle = Bitmap(overlay.group, OIUtil.getImage('score_border_bottom'))
    
    overlay.topLeft.Bottom:Set(overlay.armyLines[1].Top)
    overlay.topLeft.Left:Set(overlay.armyLines[1].leftbg.Left)
    overlay.topRight.Bottom:Set(overlay.armyLines[1].Top)
    overlay.topRight.Right:Set(overlay.armyLines[1].rightbg.Right)
    overlay.topMiddle.Bottom:Set(overlay.armyLines[1].Top)
    overlay.topMiddle.Left:Set(overlay.topLeft.Right)
    overlay.topMiddle.Right:Set(overlay.topRight.Left)
    overlay.topMiddle.Height:Set(overlay.topMiddle.BitmapHeight)
    
    overlay.bottomLeft.Top:Set(overlay.armyLines[index].Bottom)
    overlay.bottomLeft.Left:Set(overlay.armyLines[1].leftbg.Left)
    overlay.bottomRight.Top:Set(overlay.armyLines[index].Bottom)
    overlay.bottomRight.Right:Set(overlay.armyLines[1].rightbg.Right)
    overlay.bottomMiddle.Top:Set(overlay.bottomRight.Top)
    overlay.bottomMiddle.Right:Set(overlay.bottomRight.Left)
    overlay.bottomMiddle.Left:Set(overlay.bottomLeft.Right)
    
    
    overlay.unitsBG = Bitmap(overlay.group, UIUtil.UIFile('/dialogs/score-overlay/tank_bmp.dds'))
    LayoutHelpers.AtLeftTopIn(overlay.unitsBG, overlay.bottomLeft, 0, 0)
    
    overlay.units = UIUtil.CreateText(overlay.group, "", 12, UIUtil.bodyFont)
    overlay.units:SetColor('ffff9900')
    overlay.units:SetDropShadow(true)
    LayoutHelpers.RightOf(overlay.units, overlay.unitsBG)
    
    overlay.timeBG = Bitmap(overlay.group, UIUtil.UIFile('/dialogs/score-overlay/clock_bmp.dds'))
    LayoutHelpers.AtRightTopIn(overlay.timeBG, overlay.topRight, -3, -3)
#    overlay.timeBG:SetRenderPass(UIUtil.UIRP_PostGlow)
#    overlay.timeBG.Depth:Set(100)
#    overlay.timeBG.HandleEvent = function(self, event)
#        if event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
#            walltime = not walltime
#            Prefs.SetToCurrentProfile("scorewalltime", walltime)
#            if walltime then    
#                timeThread = ForkThread(UpdateTime)
#            else
#                KillThread(timeThread)
#            end
#        end
#    end
    
    overlay.time = UIUtil.CreateText(overlay.timeBG, "", 12, UIUtil.bodyFont)
    LayoutHelpers.LeftOf(overlay.time, overlay.timeBG)
    overlay.time.Top:Set(function() return overlay.timeBG.Top() + 4 end)
    overlay.time:SetDropShadow(true)
    overlay.time:SetColor('ffff9900')
    
    overlay.group:DisableHitTest(true)
    overlay.group:Hide()
    overlay.timeBG:Hide()
    overlay.group:SetNeedsFrameUpdate(true)
    overlay.group.OnFrame = function(self, deltatime)
        if Prefs.GetFromCurrentProfile("scoreoverlay") != nil and Prefs.GetFromCurrentProfile("scoreoverlay") != false then
            overlay.group:Show()
            overlay.timeBG:Show()
        end
        overlay.group:SetNeedsFrameUpdate(false)
    end
    
    GameMain.AddBeatFunction(_BeatFunction)
    
#    if walltime then    
#        timeThread = ForkThread(UpdateTime)
#    end
    
    overlay.group.OnDestroy = function(self)
        GameMain.RemoveBeatFunction(_BeatFunction)
    end
end


function _BeatFunction()
    if overlay.group:IsHidden() then
        overlay.timeBG:Hide()
        overlay.group:Hide()
    else
        overlay.time:SetText(GetGameTime())
        local armiesInfo = GetArmiesTable()
        if armiesInfo then
            focusArmy = armiesInfo.focusArmy
            local sortedFocus = 0
            local sortedTable = {}
            if currentScores != nil then
                local index = 1
                for i, v in armiesInfo.armiesTable do
                    if v.showScore and not v.civilian then
                        sortedTable[index] = v
                        sortedTable[index].ID = i
                        sortedTable[index].score = currentScores[i].general.score or 0
                        if focusArmy == index then
                            if currentScores[i].general.currentunits.count and currentScores[i].general.currentcap.count then
                                overlay.units:SetText(string.format("%d / %d", 
                                    currentScores[i].general.currentunits.count, 
                                    currentScores[i].general.currentcap.count))
                            end
                            sortedTable[index].focus = true
                        else
                            sortedTable[index].focus = false
                        end
                        index = index + 1
                    end
                end
                table.sort(sortedTable, function(table1, table2)
                    if table1.score >= table2.score then
                        return false
                    else
                        return true
                    end
                end)
            end
            for i, v in sortedTable do
                local index = table.getsize(sortedTable) - i + 1
                overlay.armyLines[index].score:SetText(v.score)
                if v.outOfGame and not overlay.armyLines[index].OOG then
                    overlay.armyLines[index].OOG = true
                    overlay.armyLines[index].faction:SetTexture(UIUtil.UIFile('/game/unit-over/icon-skull_bmp.dds'))
                    overlay.armyLines[index].teamcolor:Hide()
                    overlay.armyLines[index].playername:SetColor('ffa0a0a0')
                    overlay.armyLines[index].score:SetColor("ffa0a0a0")
                    overlay.armyLines[index].playername:SetText(v.nickname)
                end
                if overlay.armyLines[index].ID != v.ID then
                    overlay.armyLines[index].ID = v.ID
                    if not v.outOfGame then
                        overlay.armyLines[index].OOG = false
                        SetBackgroundColor(overlay.armyLines[index], v.focus)
                        overlay.armyLines[index].playername:SetText(v.nickname)
                        overlay.armyLines[index].teamcolor:Show()
                        overlay.armyLines[index].teamcolor:SetSolidColor(v.color)
                        overlay.armyLines[index].faction:SetTexture(UIUtil.UIFile(UIUtil.GetFactionIcon(v.faction)))
                    end
                end
            end
            local maxPlayerWidth = 0
            local maxScoreWidth = 0
            local index = 0
            for i, v in overlay.armyLines do
                index = i
                if v.playername.Width() > maxPlayerWidth then
                    maxPlayerWidth = v.playername.Width()
                end
                if v.score.Width() > maxScoreWidth then
                	maxScoreWidth = v.score.Width()
            	end
            end
            for i, v in overlay.armyLines do
                v.nameBG.Width:Set(math.max(maxPlayerWidth + 10, 70))
                v.scoreBG.Width:Set(math.max(maxScoreWidth + 10, 20))
                --v.playername.Right:Set(function() return overlay.armyLines[index].score.Left() - 10 end)
                --v.score.Left:Set(function() return overlay.armyLines[index].playername.Right() + 10 end)
            end
        end
    end
end



local unitColor = 'red'

function SetBackgroundColor(control, focus)
    if focus then
        control.playername:SetColor("FFFFFFFF")
        control.score:SetColor("FFFFFFFF")
        --control.nameBG:SetSolidColor("70FFFFFF")
    else
        control.playername:SetColor("FFFFFFFF")
        control.score:SetColor("FFFFFFFF")
        --control.nameBG:SetSolidColor("70000000")
    end
end


