

local OIUtil = import('/mods/OllosInterface/libs/oiutil.lua')
            
function SetSelectionIcons(sortedUnitTable)
    ClearQueueGrid()
    import('/lua/ui/game/construction.lua').SetConstructionState(true)
    import('/lua/ui/game/queue.lua').SetQueueState(true)
    local numTypes = table.getn(sortedUnitTable)
    local numIcons = 0
    if numTypes and numTypes > 0 then
        local iconBPID = nil
        local cols, rows = controls.iconGrid:GetDimensions()
        local maxcols, maxrows = controls.iconGrid:GetVisible()
        numIcons = table.getn(sortedUnitTable)
        
        PopulateQueueGrid(numIcons)
        
        local i = 1
        local curcol = 1
        for index, v in sortedUnitTable do
            local icon = controls.iconGrid:GetItem(i, curcol)
            if not icon then
                WARN("Selection Manager: SetSelectionIcons attempt to get item ", i, " but it's not there.")
            end
            local blueprint = __blueprints[v[1]]
            local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
            icon.unitIcon:SetTexture(upIconName)
            icon.text:SetText(v[2])
            icon._position = i
            icon._unitType = v[1]
            icon._units = v[3]
            if i >= maxcols then
                i = 1
                curcol = curcol + 1
            else
                i = i + 1
            end
            
            --()((()-- add strategic icon
            OIUtil.addStrategicIconTo(icon.unitIcon, blueprint)
            --()((()--
            
            icon.HandleEvent = function(self, event)
                if event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
                    if event.Modifiers.Left then
                        SelectUnits(self._units)
                    else
                        local ind = 1
                        local newUnitSelectTable = {}
                        for i, v in sortedUnitTable do
                            if self._unitType != v[1] then
                                for y, z in v[3] do
                                    newUnitSelectTable[ind] = z
                                    ind = ind + 1
                                end
                            end
                        end
                        SelectUnits(newUnitSelectTable)
                    end
                elseif event.Type == 'MouseEnter' then
                    local blueprint = __blueprints[self._unitType]
                    --import('/lua/modules/ui/game/unitviewDetail.lua').Show(blueprint, nil, self._unitType)
                    local tempText = blueprint.Description
                    local tempDesc = ''
                    if blueprint.General.UnitName then
                        tempText = blueprint.General.UnitName
                        tempDesc = blueprint.Description
                    end
                    Tooltip.CreateMouseoverDisplay(self.unitIcon, {text = tempText, body = tempDesc}, 0, true)
                elseif event.Type == 'MouseExit' then
                    --import('/lua/modules/ui/game/unitviewDetail.lua').Hide()
                    Tooltip.DestroyMouseoverDisplay()
                end
                Bitmap.HandleEvent(self, event)
            end
        end
    else
    	--import('/lua/ui/game/construction.lua').disableTechButtons()
        controls.scrollup:Disable()
        controls.scrolldown:Disable()
        if controls.panelmin.Mini then
            controls.panelmin.Mini(true)
        end
    end
        
    controls.iconGrid:EndBatch()
    --()((()-- hide upgrade button
    import('/lua/ui/game/construction.lua').disableTechButtons()
    --()((()--
end

function SetAttachedIcons(attachedUnits)
    ClearQueueGrid()
    local numTypes = table.getn(attachedUnits)
    local numIcons = 0
    if numTypes then
        local iconBPID = nil
        local cols, rows = controls.iconGrid:GetDimensions()
        local maxcols, maxrows = controls.iconGrid:GetVisible()
        numIcons = table.getn(attachedUnits)
        
        PopulateQueueGrid(numIcons, true)
        
        local i = 1
        local curcol = 1
        for index, v in attachedUnits do
            local blueprintID = v:GetBlueprint().BlueprintId
            local blueprint = __blueprints[blueprintID]
            local icon = controls.iconGrid:GetItem(i, curcol)
            local userUnit = v
            if not icon then
                WARN("Selection Manager: SetSelectionIcons attempt to get item ", i, ":", curcol, " but it's not there.")
            end
            local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
            icon.unitIcon:SetTexture(upIconName)
            icon.unitIcon:SetNewTextures(upIconName, downIconName, overIconName, iconName)
            icon.unitIcon:SetUpAltButtons( downIconName, downIconName, downIconName, downIconName )
            icon.unitIcon._unitType = blueprintID
            if i >= maxcols then
                i = 1
                curcol = curcol + 1
            else
                i = i + 1
            end
            
            icon.unitIcon.OnClick = function(self, modifiers)
                if modifiers.Left then
                    -- Toggling selection of the entity
                    self:OnAltToggle()
    
                    -- Add or Remove the entity to the session selection
                    if self.mAltToggledFlag then
                        AddToSessionExtraSelectList(userUnit)
                    else
                        RemoveFromSessionExtraSelectList(userUnit)
                    end
                end
            end
            icon.unitIcon.HandleEvent = function(self, event)
                if event.Type == 'MouseEnter' then
                    local blueprint = __blueprints[self._unitType]
                    --import('/lua/modules/ui/game/unitviewDetail.lua').Show(blueprint, nil, self._unitType)
                    local tempText = blueprint.Description
                    local tempDesc = ''
                    if blueprint.General.UnitName then
                        tempText = blueprint.General.UnitName
                        tempDesc = blueprint.Description
                    end
                    Tooltip.CreateMouseoverDisplay(self, {text = tempText, body = tempDesc}, 0, true)
                elseif event.Type == 'MouseExit' then
                    --import('/lua/modules/ui/game/unitviewDetail.lua').Hide()
                    Tooltip.DestroyMouseoverDisplay()
                end
                Button.HandleEvent(self, event)
            end
        end
    end
    controls.iconGrid:EndBatch()
end


