


--local LazyVar = import('/lua/lazyvar.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')

local Group = import('/lua/maui/group.lua').Group
local Text = import('/lua/maui/text.lua').Text
local MultiLineText = import('/lua/maui/multilinetext.lua').MultiLineText
local Button = import('/lua/maui/button.lua').Button
local Edit = import('/lua/maui/edit.lua').Edit
local Checkbox = import('/lua/maui/Checkbox.lua').Checkbox
local Scrollbar = import('/lua/maui/scrollbar.lua').Scrollbar
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Cursor = import('/lua/maui/cursor.lua').Cursor
local Border = import('/lua/maui/border.lua').Border
local ItemList = import('/lua/maui/itemlist.lua').ItemList

--local Options = import('/lua/ui/game/options.lua')
local UIUtil = import('/lua/ui/uiutil.lua')
local Prefs = import('/lua/user/prefs.lua')





--------------------------------------------------------------------------------
--==============================================================================
--                          basic skinning stuff
--==============================================================================
--------------------------------------------------------------------------------

local aSkinnableImages = {
	
	['orders_horizontal'] =			'/backgrounds/orders.dds',
	['replay_horizontal'] =			'/backgrounds/replay.dds',
	
	['selection_h_left'] =			'/backgrounds/selection_left.dds',
	['selection_h_bg'] =			'/backgrounds/selection_middle.dds',
	['selection_h_right'] =			'/backgrounds/selection_right.dds',
	
	['buildqueue_h_left'] =			'/backgrounds/buildqueue_left.dds',
	['buildqueue_h_bg']	=			'/backgrounds/buildqueue_middle.dds',
	['buildqueue_h_right'] =		'/backgrounds/buildqueue_right.dds',
	
	['queue_h_left'] =				'/backgrounds/queue_left.dds',
	['queue_h_bg']	=				'/backgrounds/queue_middle.dds',
	['queue_h_right'] =				'/backgrounds/queue_right.dds',
	
	['build_infinite'] =			'/buttons/build_infinite.dds',
	['build_infinite_mo'] =			'/buttons/build_infinite_mo.dds',
	['build_infinite_dis'] =		'/buttons/build_infinite_dis.dds',
	['build_infinite_sel'] =		'/buttons/build_infinite_sel.dds',
	['build_infinite_sel_mo'] =		'/buttons/build_infinite_sel_mo.dds',
	['build_pause'] =				'/buttons/build_pause.dds',			
	['build_pause_mo'] =			'/buttons/build_pause_mo.dds',			
	['build_pause_dis'] =			'/buttons/build_pause_dis.dds',	
	['build_pause_sel'] =			'/buttons/build_pause_sel.dds',
	['build_pause_sel_mo'] =		'/buttons/build_pause_sel_mo.dds',
	
	['build_h_left'] =				'/backgrounds/build_left.dds',
	['build_h_bg']	=				'/backgrounds/build_middle.dds',
	['build_h_right'] =				'/backgrounds/build_right.dds',
	['build_h_top'] =				'/backgrounds/build_top.dds',
	
	['t1'] = 						'/buttons/tech123.dds',
	['t1_mo'] = 					'/buttons/tech123_mo.dds',
	['t1_dis'] = 					'/buttons/tech123_dis.dds',
	['t2'] = 						'/buttons/tech123.dds',
	['t2_mo'] = 					'/buttons/tech123_mo.dds',
	['t2_dis'] = 					'/buttons/tech123_dis.dds',
	['t3'] = 						'/buttons/tech123.dds',
	['t3_mo'] = 					'/buttons/tech123_mo.dds',
	['t3_dis'] = 					'/buttons/tech123_dis.dds',
	['t4'] = 						'/buttons/tech4.dds',
	['t4_mo'] = 					'/buttons/tech4_mo.dds',
	['t4_dis'] = 					'/buttons/tech4_dis.dds',
	['upgrade'] =					'/buttons/upgrade.dds',
	['upgrade_mo'] = 				'/buttons/upgrade_mo.dds',
	['enhancements_removeicon'] =	'/icons_generic/remove.dds',
	['icon_t1'] =					'/icons_generic/tech1.dds',
	['icon_t2'] =					'/icons_generic/tech2.dds',
	['icon_t3'] =					'/icons_generic/tech3.dds',
	['icon_t4'] =					'/icons_generic/tech4.dds',
	
	['scroll_left'] = 				'/buttons/scroll_left.dds',
	['scroll_left_mo'] = 			'/buttons/scroll_left_mo.dds',
	['scroll_left_dis'] = 			'/buttons/scroll_left_dis.dds',
	['scroll_right'] =				'/buttons/scroll_right.dds',
	['scroll_right_mo'] =			'/buttons/scroll_right_mo.dds',
	['scroll_right_dis'] =			'/buttons/scroll_right_dis.dds',
	['sm_scroll_left'] = 			'/buttons/selman_scroll_left.dds',
	['sm_scroll_left_mo'] = 		'/buttons/selman_scroll_left_mo.dds',
	['sm_scroll_left_dis'] = 		'/buttons/selman_scroll_left_dis.dds',
	['sm_scroll_right'] =			'/buttons/selman_scroll_right.dds',
	['sm_scroll_right_mo'] =		'/buttons/selman_scroll_right_mo.dds',
	['sm_scroll_right_dis'] =		'/buttons/selman_scroll_right_dis.dds',
	
	['overlays_h_bg'] =				'/backgrounds/overlays_bg.dds',
	
	['minimap_h_bg'] =				'/backgrounds/minimap_bg.dds',
	['minimap_h_fg'] =				'/backgrounds/minimap_fg.dds',
	['minimap_glow'] =				'/backgrounds/minimap_glow.dds',
	
	['economy_left'] =				'/backgrounds/resources_left_bg.dds',
	['economy_right'] =				'/backgrounds/resources_right_bg.dds',
	['economy_bg'] =				'/backgrounds/resources_bg.dds',
	['resource_rate'] =				'/resources/rate.dds',
	['resource_rate_mo'] =			'/resources/rate_mo.dds',
	['resource_rate_advanced'] =	'/resources/rate_advanced.dds',
	['resource_rate_advanced_mo'] = '/resources/rate_advanced_mo.dds',
	['economy_warning_negative'] =	'/resources/warning_negative.dds',
	['economy_warning_positive'] =	'/resources/warning_positive.dds',
	['mass_bar'] =					'/resources/mass_bar.dds',
	['mass_bar_bg'] =				'/resources/mass_bar_bg.dds',
	['mass_label'] =				'/resources/mass_label.dds',
	['energy_bar'] =				'/resources/energy_bar.dds',
	['energy_bar_bg'] =				'/resources/energy_bar_bg.dds',
	['energy_label'] =				'/resources/energy_label.dds',
	
	['options_bg'] =				'/backgrounds/options_bg.dds',
	['play'] = 						'/icons_options/play.dds',
	['play_mo'] =					'/icons_options/play_mo.dds',
	['pause'] = 					'/icons_options/pause.dds',
	['pause_mo'] =					'/icons_options/pause_mo.dds',
	['inbox'] = 					'/icons_options/inbox.dds',
	['inbox_mo'] =					'/icons_options/inbox_mo.dds',
	['objectives'] = 				'/icons_options/objectives.dds',
	['objectives_mo'] =				'/icons_options/objectives_mo.dds',
	['exit_menu'] = 				'/icons_options/menu.dds',
	['exit_menu_mo'] =				'/icons_options/menu_mo.dds',
	['diplomacy'] = 				'/icons_options/diplomacy.dds',
	['diplomacy_mo'] =				'/icons_options/diplomacy_mo.dds',
	['score'] = 					'/icons_options/score.dds',
	['score_mo'] =					'/icons_options/score_mo.dds',
	
	['unitview_bg'] =				'/overlays/unitview_bg.dds',
	['unitview_build_bg'] =			'/overlays/unitview_build_bg.dds',
	
	['unitdetails_bg'] =			'/overlays/unitdetails_bg.dds',
	
	['shadow_top'] =				'/window_shadows/top.dds',
	['shadow_topleft'] =			'/window_shadows/topleft.dds',
	['shadow_topright'] =			'/window_shadows/topright.dds',
	['shadow_bottom'] =				'/window_shadows/bottom.dds',
	['shadow_bottomleft'] =			'/window_shadows/bottomleft.dds',
	['shadow_bottomright'] =		'/window_shadows/bottomright.dds',
	['shadow_left'] =				'/window_shadows/left.dds',
	['shadow_right'] =				'/window_shadows/right.dds',
	
	['text_top'] =					'/textbox/top.dds',
	['text_topleft'] =				'/textbox/topleft.dds',
	['text_topright'] =				'/textbox/topright.dds',
	['text_bottom'] =				'/textbox/bottom.dds',
	['text_bottomleft'] =			'/textbox/bottomleft.dds',
	['text_bottomright'] =			'/textbox/bottomright.dds',
	['text_left'] =					'/textbox/left.dds',
	['text_right'] =				'/textbox/right.dds',
	['text_bg'] =					'/textbox/bg.dds',
	
	['text_fulltop'] =				'/textbox/fulltop.dds',
	['text_fullmiddle'] =			'/textbox/fullmiddle.dds',
	['text_fullbottom'] =			'/textbox/fullbottom.dds',
	
	['avatar_bg'] =					'/overlays/avatar_bg.dds',
	['avatar_tab'] =				'/overlays/avatar_tab.dds',
	['avatar_glow'] =				'/overlays/avatar_glow.dds',
	['toggleavatar_on'] =			'/buttons/avatartoggle_on.dds',
	['toggleavatar_on_mo'] =		'/buttons/avatartoggle_on_mo.dds',
	['toggleavatar_off'] =			'/buttons/avatartoggle_off.dds',
	['toggleavatar_off_mo'] =		'/buttons/avatartoggle_off_mo.dds',
	
	['score_line_left'] = 			'/overlays/score_line_left.dds',
	['score_line_right'] = 			'/overlays/score_line_right.dds',
	['score_line_middle'] =			'/overlays/score_line_middle.dds',
	['score_border_topleft'] = 		'/overlays/score_border_topleft.dds',
	['score_border_topright'] = 	'/overlays/score_border_topright.dds',
	['score_border_top'] = 			'/overlays/score_border_top.dds',
	['score_border_bottomleft'] = 	'/overlays/score_border_bottomleft.dds',
	['score_border_bottomright'] = 	'/overlays/score_border_bottomright.dds',
	['score_border_bottom'] = 		'/overlays/score_border_bottom.dds',
	
	--[[
	[''] = '',
	[''] = '',
	[''] = '',
	[''] = '',
	[''] = '',
	[''] = '',
	[''] = '',
	--]]
	
}

local aSkinnableColors = {

	['default'] = {
		['economy_income'] =			'00A200',
		['economy_expense'] =			'E52525',
		['economy_rate_green'] =		'00A200',
		['economy_rate_yellow'] =		'E8F426',
		['economy_rate_red'] =			'E52525',
	}
	
}

local aSkinnableValues = {
	
	['default'] = {
		['economy_barstep'] =			7,
	}
	
}

local aSkins = {
	'default',
	'aeon',
	'cybran',
}

local iCurrentSkin = 1

--------------------------------------------------------------------------------
-- get current skin
--
function getCurrentSkin()

	return aSkins[iCurrentSkin]
	
end

--------------------------------------------------------------------------------
--
--
function setCurrentSkin(iSkin)
	
	iCurrentSkin = iSkin
	
end

--------------------------------------------------------------------------------
--
--
function rotateSkin(iDirection)
	
	local iNumSkins = table.getn(aSkins)
	local iNextSkin = iCurrentSkin + iDirection
    if iNextSkin > iNumSkins then iNextSkin = 1 end
    if iNextSkin < 1 then iNextSkin = iNumSkins end
	setCurrentSkin(iNextSkin)
    UIUtil.SetCurrentLayout('bottom')
	import('/lua/ui/game/avatars.lua').applySkin()
	
end

--------------------------------------------------------------------------------
-- get (non)skinnable image from lib
--
function getImage(sID)
	
	if (sID == 'blank') then
		return '/mods/OllosInterface/textures/blank.dds'
	end
	
	if not aSkinnableImages[sID] then
		LOG('OI:getImage(): image not found - "' .. sID .. '"')
	end
	
	return '/mods/OllosInterface/textures/skin_' .. getCurrentSkin() .. aSkinnableImages[sID]
	
end

--------------------------------------------------------------------------------
-- get (non)skinnable color from lib
--
function getColor(sID)

	if (aSkinnableColors[getCurrentSkin()][sID]) then
		return aSkinnableColors[getCurrentSkin()][sID]
	elseif (aSkinnableColors['default'][sID]) then
		return aSkinnableColors['default'][sID]
	else
		LOG('OI:getColor(): color not found: ' .. sID)
		return 'white'
	end

end

--------------------------------------------------------------------------------
-- get (non)skinnable value from lib
--
function getValue(sID)

	if (aSkinnableValues[getCurrentSkin()][sID]) then
		return aSkinnableValues[getCurrentSkin()][sID]
	elseif (aSkinnableValues['default'][sID]) then
		return aSkinnableValues['default'][sID]
	else
		LOG('OI:getValue(): value not found: ' .. sID)
		return 0
	end

end















--------------------------------------------------------------------------------
--==============================================================================
--                tools to add bitmaps to controls and such
--==============================================================================
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- add a strategic icon to parent
--
function addStrategicIconTo(oParent, bpUnit)
	
	if Prefs.GetOption('oi_straticons_activated') == 1 then
		if not oParent.bmStratIcon then
			local sTexture = false
			local sSet = Prefs.GetOption('oi_straticons_set')
			local sIconsPath = '/mods/OllosInterface/textures/icons_strategic/' .. sSet .. '/'
			local aStratIcons = import(sIconsPath .. 'straticons.lua')
			if aStratIcons.aSpecific[bpUnit.BlueprintId] then
				sTexture = sIconsPath .. aStratIcons.aSpecific[bpUnit.BlueprintId] .. '.dds'
			elseif aStratIcons.aTranslation[bpUnit.StrategicIconName] then
		    	sTexture = sIconsPath .. aStratIcons.aTranslation[bpUnit.StrategicIconName] .. '.dds'
		    end
		    if sTexture then
		    	bmStratIcon = Bitmap(oParent)
		    	bmStratIcon:SetTexture(sTexture)
		    	LayoutHelpers.AtRightBottomIn(bmStratIcon, oParent, aStratIcons.iOffsetX, aStratIcons.iOffsetY)
		    	bmStratIcon:DisableHitTest()
		    	bmStratIcon:SetAlpha(Prefs.GetOption('oi_straticons_opacity') / 100)
		    	oParent.bmStratIcon = bmStratIcon
		    end
	    end
	end
	
end

function removeStrategicIconFrom(oParent)
	
	if oParent.bmStratIcon then
		oParent.bmStratIcon:Destroy()
		oParent.bmStratIcon = nil
	end
	
end

function updateStrategicIcons()

    WaitSeconds(1)
    
    local queueGrid = import('/lua/ui/game/queue.lua').controls.queueGrid
	--LOG('updateStrategicIcons')
	for iIndex, oItem in queueGrid._items[1] do
		removeStrategicIconFrom(oItem.unitIcon)
		local sID = oItem._blueprintId
		local bpUnit = __blueprints[sID]
		addStrategicIconTo(oItem.unitIcon, bpUnit)
	end
	
end

--------------------------------------------------------------------------------
-- add a 'remove' icon to parent
--
function addRemoveIconTo(oParent)
	
	if not oParent.bmRemoveIcon then
		bmRemoveIcon = Bitmap(oParent)
    	bmRemoveIcon:SetTexture(getImage('enhancements_removeicon'))
    	LayoutHelpers.AtLeftTopIn(bmRemoveIcon, oParent, 8, 8)
    	bmRemoveIcon:DisableHitTest()
    	oParent.bmRemoveIcon = bmRemoveIcon
    end
	
end
















--------------------------------------------------------------------------------
--==============================================================================
--                           cloned functions
--==============================================================================
--------------------------------------------------------------------------------

--* return a button set up with a text overlay and a click sound
function CreateButton(parent, up, down, over, disabled, label, pointSize, textOffsetVert, textOffsetHorz, clickCue, rolloverCue)
    textOffsetVert = textOffsetVert or 0
    textOffsetHorz = textOffsetHorz or 0
    if clickCue == "NO_SOUND" then
        clickCue = nil
    else
        clickCue = clickCue or "UI_Menu_MouseDown_Sml"
    end
    if rolloverCue == "NO_SOUND" then
        rolloverCue = nil
    else
        rolloverCue = rolloverCue or "UI_Menu_Rollover_Sml"
    end
    if type(up) == 'string' then
        up = getImage(up)
    end
    if type(down) == 'string' then
        down = getImage(down)
    end
    if type(over) == 'string' then
        over = getImage(over)
    end
    if type(disabled) == 'string' then
        disabled = getImage(disabled)
    end

    local button = Button(parent, up, down, over, disabled, clickCue, rolloverCue)
    button:UseAlphaHitTest(true)

    if label and pointSize then
        button.label = UIUtil.CreateText(button, label, pointSize)
        LayoutHelpers.AtCenterIn(button.label, button, textOffsetVert, textOffsetHorz)
        button.label:DisableHitTest()

        -- if text exists, set up to grey it out
        button.OnDisable = function(self)
            Button.OnDisable(self)
            button.label:SetColor(UIUtil.disabledColor)
        end

        button.OnEnable = function(self)
            Button.OnEnable(self)
            button.label:SetColor(UIUtil.fontColor)
        end
        button.OnRolloverEvent = function(self, event)
            if event == 'enter' then
                button.label:SetColor(UIUtil.fontOverColor)
            elseif event == 'exit' then
                button.label:SetColor(UIUtil.fontColor)
            elseif event == 'down' then
                button.label:SetColor(UIUtil.fontDownColor)
            end
        end
    end

    return button
end

function SetNewButtonTextures(button, up, down, over, disabled)
    -- if strings passed in, make them skinnables, otherwise assume they are already skinnables
    if type(up) == 'string' then
        up = getImage(up)
    end
    if type(down) == 'string' then
        down = getImage(down)
    end
    if type(over) == 'string' then
        over = getImage(over)
    end
    if type(disabled) == 'string' then
        disabled = getImage(disabled)
    end

    button:SetNewTextures(up, down, over, disabled)
end


--* create a button with standardized texture names
--* given a path and button name prefix, generates the four button asset file names according to the naming convention
function CreateButtonStd(parent, name, label, pointSize, textOffsetVert, textOffsetHorz, clickCue, rolloverCue)
    return CreateButton(parent
        , name
        , name .. "_mo"
        , name .. "_mo"
        , name
        , label
        , pointSize
        , textOffsetVert
        , textOffsetHorz
        , clickCue
        , rolloverCue
        )
end


function CreateButtonFullPath(parent, up, down, over, disabled, label, pointSize, textOffsetVert, textOffsetHorz, clickCue, rolloverCue)
    textOffsetVert = textOffsetVert or 0
    textOffsetHorz = textOffsetHorz or 0
    if clickCue == "NO_SOUND" then
        clickCue = nil
    else
        clickCue = clickCue or "UI_Menu_MouseDown_Sml"
    end
    if rolloverCue == "NO_SOUND" then
        rolloverCue = nil
    else
        rolloverCue = rolloverCue or "UI_Menu_Rollover_Sml"
    end

    local button = Button(parent, up, down, over, disabled, clickCue, rolloverCue)
    button:UseAlphaHitTest(true)

    if label and pointSize then
        button.label = CreateText(button, label, pointSize)
        LayoutHelpers.AtCenterIn(button.label, button, textOffsetVert, textOffsetHorz)
        button.label:DisableHitTest()

        -- if text exists, set up to grey it out
        button.OnDisable = function(self)
            Button.OnDisable(self)
            button.label:SetColor(disabledColor)
        end

        button.OnEnable = function(self)
            Button.OnEnable(self)
            button.label:SetColor(fontColor)
        end
        button.OnRolloverEvent = function(self, event)
            if event == 'enter' then
                button.label:SetColor(fontOverColor)
            elseif event == 'exit' then
                button.label:SetColor(fontColor)
            elseif event == 'down' then
                button.label:SetColor(fontDownColor)
            end
        end
    end

    return button
end












--------------------------------------------------------------------------------
--==============================================================================
--                           OI Layout Engine
--==============================================================================
--------------------------------------------------------------------------------

gameParent = false
sCurrentLayout = 'none'
aCurrentLayout = { }
aKnownLayouts = { 
	'test',
}

--------------------------------------------------------------------------------
-- initialize Layout engine
--
function initLayout()
	
	LOG('OI:setLayout(): Initializing gameParent')
	--local gameParent = import('/lua/modules/ui/game/gamemain.lua').GetGameParent()
	gameParent = import('/lua/ui/game/borders.lua').controls.borderGroup:GetParent()
	
end

--------------------------------------------------------------------------------
-- rotatelayout
--
function rotateLayout()

	if sCurrentLayout == 'none' then
		setLayout('test')
		sCurrentLayout = 'test'
	else
		destroyLayout()
		sCurrentLayout = 'none'
	end

end

--------------------------------------------------------------------------------
-- destroy all previously created UIElements
--
function destroyLayout()
	
	if table.getsize(aCurrentLayout) != 0 then
		LOG('OI:setLayout(): destroying existing controls')
		for sName, oUIElement in aCurrentLayout do
			oUIElement:Destroy()
		end
		aCurrentLayout = { }
	end

end

--------------------------------------------------------------------------------
-- set complete Layout
--
function setLayout(sLayout)
	
	local aLayoutInfo = import('/mods/OllosInterface/layouts/' .. sLayout .. '.lua').layout
	LOG('OI:setLayout(): setting layout "' .. sLayout .. '"')
	applyLayout(aLayoutInfo, gameParent)
	
end

--------------------------------------------------------------------------------
-- apply a layout to a subtree
--
function applyLayout(aLayoutInfo, oParent)
	
	if not oParent then
		WARN('OI:applyLayout(): parent object is not set!')
		return
	end
	
	oParent['aChildren'] = { }
	
	for sName, aDetails in aLayoutInfo do
		
		if sName != 'appearance' then
			
			LOG('OI:applyLayout(): creating control "' .. sName .. '"')
			
			local oCurrentControl = false
			local sType = aDetails.appearance.type
			
			-- create object ---------------------------------------------------
			if  sType == 'Bitmap' then
				
				oCurrentControl = Bitmap(oParent)
				local sTexture = aDetails.appearance.texture[1] or false
				local iBorder = aDetails.appearance.texture[2] or 1
				if sTexture then
					oCurrentControl:SetTexture(sTexture, iBorder)
				end
				
				aCurrentLayout[sName] = oCurrentControl
				oParent.aChildren[sName] = oCurrentControl
				
			elseif sType == 'Button' then
				
				local aTextures = aDetails.appearance.textures
				oCurrentControl = Button(oParent, aTextures[1], aTextures[2], aTextures[3], aTextures[4])
				--oCurrentControl:SetNewTextures(aTextures[1], aTextures[2], aTextures[3], aTextures[4])
				
				aCurrentLayout[sName] = oCurrentControl
				oParent.aChildren[sName] = oCurrentControl
				
			else
				WARN('OI:applyLayout(): unrecognized control type "' .. sType .. '"')
			end
			
			-- assign name to control ------------------------------------------
			oCurrentControl:SetName(sName)
			
			-- position control ------------------------------------------------
			if aDetails.appearance.Left then
				
				local oSubject = oParent
				if aDetails.appearance.Left[1] != nil then
					oSubject = aCurrentLayout[ aDetails.appearance.Left[1] ]
				end
				
				local sSubjectBorder = aDetails.appearance.Left[2]
				local iOffset = aDetails.appearance.Left[3] or 0
				
				oCurrentControl.Left:Set(function() return oSubject[sSubjectBorder]() + iOffset end)
				
			end
			
			if aDetails.appearance.Top then
				
				local oSubject = oParent
				if aDetails.appearance.Top[1] != nil then
					oSubject = aCurrentLayout[ aDetails.appearance.Top[1] ]
				end
				
				local sSubjectBorder = aDetails.appearance.Top[2]
				local iOffset = aDetails.appearance.Top[3] or 0
				
				oCurrentControl.Top:Set(function() return oSubject[sSubjectBorder]() + iOffset end)
				
			end
			
			if aDetails.appearance.AtLeftTopIn then
				
				local oSubject = oParent
				if aDetails.appearance.AtLeftTopIn[1] != nil then
					oSubject = aCurrentLayout[ aDetails.appearance.AtLeftTopIn[1] ]
				end
				
				local sSubjectBorder = aDetails.appearance.AtLeftTopIn[2]
				local iOffsetX = aDetails.appearance.AtLeftTopIn[2] or 0
				local iOffestY = aDetails.appearance.AtLeftTopIn[3] or 0
				
				LayoutHelpers.AtLeftTopIn(oCurrentControl, oSubject, iOffsetX, iOffsetY)
				
			end
			
			if aDetails.appearance.Depth then
				
				local oSubject = oParent
				if aDetails.appearance.Top[1] != nil then
					oSubject = aCurrentLayout[ aDetails.appearance.Depth[1] ]
				end
				
				local iOffset = aDetails.appearance.Depth[2] or 1
				
				oCurrentControl.Depth:Set(function() return oSubject.Depth() + iOffset end)
				
			end
			
			-- recurse subcontrols ---------------------------------------------
			applyLayout(aDetails, oCurrentControl)
			
		end
		
	end

end


